/**
 * Created by zhaojh on 2017/11/16.
 */

function dtSelector(options) {
    var params = Object.assign({
        parentId: null,
        itemCount: 2,
        selectColor: '#FD8C26',
        selectBgColor: '#EFEFF5',
        normalColor: '#000',
        cover: true,
        // linkage: true,  // 是否联动，默认是联动的
        height: 40,
        maxShowCount: 8,
        screenHeight: 0,
        callback: function(indexes) {}
    }, options);

    var root;
    if (params.parentId) {
        root = document.getElementById(params.parentId);
    } else {
        root = document.body;
    }
    var shadow = document.createElement('div');
    shadow.className = 'dt_selector_shadow';
    shadow.setAttribute('tapmode', 'true');
    shadow.onclick = function() {
        fnCloseAllItems();
    };
    var table = document.createElement('div');
    table.className = 'dt_selector_table';

    var selector = document.createElement('div');
    selector.className = 'dt_selector_box';
    root.appendChild(selector);
    selector.style.height = params.height + 'px';

    var selectorItems = [];
    var lastItem = null;
    for (var i = 0; i < params.itemCount; i++) {
        var item = document.createElement('div');
        item.id = 'selector_item' + i;
        item.className = 'item';
        item.style.width = 100 / params.itemCount + '%';
        item.setAttribute('tapmode', 'true');
        item.onclick = function(e) {
            fnSelectorItemClick(e);
        }

        var title = document.createElement('div');
        title.className = 'title';
        title.innerHTML = '未设置';
        title.style.color = params.normalColor;
        item.appendChild(title);
        var triangle = document.createElement('div');
        triangle.className = 'triangle';
        triangle.style.borderColor = params.normalColor + ' transparent transparent';
        item.appendChild(triangle);
        selector.appendChild(item);

        var selectorItem = {
            target: item,
            isOpen: false,
            datas: [],
            sindex: 0,
            table: null,
            lastItem: lastItem,
            nextItem: null,
            tag: i,
        };
        selectorItems.push(selectorItem);
        lastItem = selectorItem;
    }

    for (var i = 0; i < params.itemCount - 1; i++) {
        var selectorItem = selectorItems[i];
        selectorItem.nextItem = selectorItems[i + 1];
    }

    var fnSelectorItemClick = function(e) {
        var selectorItem = fnGetSelectorItem(e);
        if (!selectorItem) return;
        if (selectorItem.isOpen) {
            fnCloseSelectorItem(selectorItem);
        } else {
            fnOpenSelectorItem(selectorItem);
        }
    };

    var fnGetSelectorItem = function(e) {
        var id = e.target.id;
        for (var i = 0; i < selectorItems.length; i++) {
            var t = selectorItems[i];
            if (t.target.id == id) {
                return t;
            }
        }
        return null;
    }

    var fnOpenSelectorItem = function(t) {
        fnCloseAllItems();
        if (!t.datas || t.datas.length == 0) return;
        var title = t.target.children[0];
        title.style.color = params.selectColor;
        var triangle = t.target.children[1];
        triangle.style.transform = 'rotate(180deg)';
        triangle.style.borderColor = params.selectColor + ' transparent transparent';
        t.isOpen = true;

        fnOpenTable(t);
    };

    var fnCloseSelectorItem = function(t) {
        fnCloseTable(t);
        var title = t.target.children[0];
        title.style.color = params.normalColor;
        var triangle = t.target.children[1];
        triangle.style.transform = 'rotate(0)';
        triangle.style.borderColor = params.normalColor + ' transparent transparent';
        t.isOpen = false;
    };

    var fnCloseAllItems = function() {
        for (var i = 0; i < selectorItems.length; i++) {
            var t = selectorItems[i];
            fnCloseSelectorItem(t);
        }
    };

    var fnGetSelectIndexes = function() {
        var indexes = [];
        for (var i = 0; i < selectorItems.length; i++) {
            var item = selectorItems[i];
            indexes.push(parseInt(item.sindex));
        }
        return indexes;
    };

    var fnOpenTable = function(t) {
        var cellHeight = params.height + 'px';
        document.body.appendChild(shadow);
        shadow.style.opacity = 0.1;
        shadow.style.height = (params.screenHeight - params.height) + 'px';
        shadow.style.top = cellHeight;
        document.body.appendChild(table);
        table.style.top = cellHeight;
        if (params.cover) {
            table.style.width = '100%';
            table.style.left = 0;
        } else {
            table.style.width = 100.0 / params.itemCount + '%';
            table.style.left = t.tag * (100.0 / params.itemCount) + '%';
        }

        var tableH = t.datas.length > params.maxShowCount ?
        params.maxShowCount * params.height : t.datas.length * params.height;
        table.style.height = tableH + 'px';
        for (var i = 0; i < t.datas.length; i++) {
            var data = t.datas[i];
            var cell = document.createElement('div');
            cell.style.height = cellHeight;
            cell.id = i;
            cell.className = 'cell';
            cell.setAttribute('tapmode', 'true');
            cell.onclick = function(e) {
                t.sindex = parseInt(e.target.id);
                fnSelectItem(t);
            };
            var text = document.createElement('div');
            text.className = 'text';
            text.innerHTML = data.text;
            if (i == t.sindex) {
                text.style.color = params.selectColor;
                cell.style.background = params.selectBgColor;
            }
            cell.appendChild(text);
            if (i == t.sindex && params.cover) {
                var gou = document.createElement('div');
                gou.className = 'gou';
                gou.style.borderBottomColor = params.selectColor;
                gou.style.borderLeftColor = params.selectColor;
                cell.appendChild(gou);
            }
            table.appendChild(cell);
        }
        api.parseTapmode();
    };


    var fnCloseTable = function(t) {
        if (!t.isOpen) return;
        shadow.style.opacity = 0;
        document.body.removeChild(shadow);
        while (table.childNodes.length) {
            table.removeChild(table.childNodes[0]);
        }
        document.body.removeChild(table);
    };

    // 选择项目，展开列表
    var fnSelectItem = function(t) {
        var info = t.datas[t.sindex];
        var title = t.target.children[0];
        title.innerHTML = info.text;
        var triangle = t.target.children[1];
        triangle.style.opacity = 1;
        fnRecursionSelectItem(t);
        fnCloseSelectorItem(t);
        if (params.callback) {
            params.callback(fnGetSelectIndexes());
        }
    };

    var fnRecursionSelectItem = function(t) {
        if (t.nextItem) {
            if (t.datas && t.datas.length > 0) {
                var info = t.datas[t.sindex];
                t.nextItem.datas = info.datas;
                t.nextItem.sindex = 0;
            }
            var title = t.nextItem.target.children[0];
            var triangle = t.nextItem.target.children[1];
            if (t.nextItem.datas && t.nextItem.datas.length > 0) {
                var curInfo = t.nextItem.datas[t.nextItem.sindex];
                title.innerHTML = curInfo.text;
                triangle.style.opacity = 1;
            } else {
                title.innerHTML = '';//'未设置';
                triangle.style.opacity = 0;
            }
            fnRecursionSelectItem(t.nextItem);
        }
    };

    this.reloadDatas = function(datas) {
        fnCloseAllItems();
        if (datas.length == 0) return;
        var firstItem = selectorItems[0];
        firstItem.datas = datas;
        firstItem.sindex = 0;
        fnSelectItem(firstItem);
        api.parseTapmode();
    };

    this.close = function() {
        fnCloseAllItems();
        root.removeChild(selector);
    };
}
